local addonName, addon = ...

addon:RegisterDefaultSetting("showBagBar", true)
addon:RegisterDefaultSetting("showMicroBar", true)
addon:RegisterDefaultSetting("showWatchFrame", true)
addon:RegisterDefaultSetting("macaroonInitialized", false)


--[[-----------------------------------------------------------------------------
Config button
-------------------------------------------------------------------------------]]
local configButton = CreateFrame('Button', configButton, UIParent, 'SecureHandlerClickTemplate')

configButton:SetHeight(30)
configButton:SetWidth(62)
configButton:RegisterForClicks('LeftButtonUp', 'RightButtonUp')
configButton:SetNormalTexture([[Interface\AddOns\]] .. addonName .. [[\Media\ConfigButton]])
configButton:SetHighlightTexture([[Interface\AddOns\]] .. addonName .. [[\Media\ConfigButton]])
configButton:SetPoint('BOTTOMRIGHT', addon.skin, 'TOPRIGHT', -2, 0)
	
configButton:SetAttribute('_onclick', [[
	if button == 'LeftButton' then
		local microBar = self:GetFrameRef("microBar")
		if microBar:IsVisible() then
			microBar:Hide()
		else
			microBar:Show()
		end
	else
		control:CallMethod("OpenConfig")
	end
]])

function configButton:OpenConfig()
	if addon.configPanel then 
		if addon.configPanel:IsVisible() then
			InterfaceOptionsFrame:Hide()
		else
			addon.configPanel()
		end
	end
end

configButton:SetScript('OnEnter', function(self)
	GameTooltip:SetOwner(self, 'ANCHOR_TOP', 0, 1)
	GameTooltip:AddLine("|cffeda55fLeft Click|r toggle MicroBar", 0.2, 1, 0.2)
	GameTooltip:AddLine("|cffeda55fRight Click|r Options Panel", 0.2, 1, 0.2)
	GameTooltip:Show()
end)

configButton:SetScript('OnLeave', addon.HideTooltip)

--[[-----------------------------------------------------------------------------
Bag bar
-------------------------------------------------------------------------------]]
local bagBar = CreateFrame('Frame', nil, UIParent, 'SecureActionButtonTemplate')
bagBar:SetPoint('BOTTOMRIGHT', configButton, 'TOPRIGHT', -3, 1)
bagBar:SetHeight(1)
bagBar:SetWidth(1)

addon.backpackButton:SetFrameRef("bagBar", bagBar)

local anchor = bagBar
for _, frame in ipairs({ MainMenuBarBackpackButton, CharacterBag0Slot, CharacterBag1Slot, CharacterBag2Slot, CharacterBag3Slot }) do
	frame:ClearAllPoints()
	frame:SetPoint('BOTTOMRIGHT', anchor, 'TOPRIGHT', 0, 2)
	frame:SetParent(bagBar)
	addon:LockFrame(frame)
	anchor = frame
end
MainMenuBarBackpackButton:SetScale(30/37)

KeyRingButton:SetParent(bagBar)
KeyRingButton:SetPoint('BOTTOMRIGHT', MainMenuBarBackpackButton, 'BOTTOMLEFT', -2, -2)
KeyRingButton:SetScale(23/37)
KeyRingButton:Show()
addon:LockFrame(KeyRingButton)

--[[-----------------------------------------------------------------------------
Micro buttons bar
-------------------------------------------------------------------------------]]
local microBar = CreateFrame('Frame', nil, UIParent, 'SecureActionButtonTemplate')
microBar:SetPoint('BOTTOMRIGHT', configButton, 'BOTTOMLEFT', -1, 0)
microBar:SetHeight(1)
microBar:SetWidth(1)
microBar:SetScale(33/37)


local buttons = { CharacterMicroButton, SpellbookMicroButton, TalentMicroButton, AchievementMicroButton, PVPMicroButton, LFDMicroButton, MainMenuMicroButton, HelpMicroButton }
-- removed FriendsMicroButton, QuestLogMicroButton, GuildMicroButton - simply put them back in the above list to have them appear in the MicroBar again.

local anchorFrame, anchorPoint = microBar, 'BOTTOMRIGHT'
for index = #buttons, 1, -1 do
	local button = buttons[index]
	button:SetParent(microBar)
	button.SetParent = addon.DoNothing
	button:ClearAllPoints()
	button:SetPoint('BOTTOMRIGHT', anchorFrame, anchorPoint, 1, 0)
	addon:LockFrame(button)
	anchorFrame, anchorPoint = button, 'BOTTOMLEFT'
end
microBar:SetWidth(buttons[#buttons]:GetRight() - buttons[1]:GetLeft() + 1)
microBar:SetHeight(buttons[1]:GetHeight())

configButton:SetFrameRef("microBar", microBar)
addon:HideFrame("FriendsMicroButton")
addon:HideFrame("QuestLogMicroButton")
addon:HideFrame("GuildMicroButton")

--[[-----------------------------------------------------------------------------
Initialize
-------------------------------------------------------------------------------]]
addon.RegisterEvent("Features-Initialize", 'PLAYER_ENTERING_WORLD', function(self, event)
	addon.UnregisterEvent(self, event)

	if not addon.settings.showBagBar then
		bagBar:Hide()
	end
	if not addon.settings.showMicroBar then
		microBar:Hide()
	end
	addon:InitWatchFrame()
	
	

end)

function addon:InitWatchFrame()
	if addon.settings.showWatchFrame == true then
		addon:ShowFrame(WatchFrame)
	else
		addon:HideFrame(WatchFrame)
	end
end

--[[-----------------------------------------------------------------------------
Save settings on logout
-------------------------------------------------------------------------------]]
addon.RegisterEvent("Features-SaveState", 'PLAYER_LOGOUT', function()
	addon.settings.showBagBar = bagBar:IsShown() or false
	addon.settings.showMicroBar = microBar:IsShown() or false
	if WatchFrame:IsShown() then
		addon.settings.showWatchFrame = true
	else
		addon.settings.showWatchFrame = false
	end
end)

--[[-----------------------------------------------------------------------------
Auto loot hot point
-------------------------------------------------------------------------------]]
local lootHotPoint = CreateFrame('Button', nil, UIParent)
lootHotPoint:SetPoint('TOPRIGHT', addon.skin[1], 'TOPRIGHT', -82, 0)
lootHotPoint:SetHeight(15)
lootHotPoint:SetWidth(15)
lootHotPoint:RegisterForClicks('LeftButtonUp', 'RightButtonUp')

lootHotPoint:SetScript('OnClick', function(self, button)
	if button == 'LeftButton' and not IsControlKeyDown() then
		if GetCVarBool('autoLootDefault') then
			SetCVar('autoLootDefault', "0")
			UIErrorsFrame:AddMessage("AutoLoot Off", 1.0, 0.0, 0.0, 53, 3);
		else
			SetCVar('autoLootDefault', "1")
			UIErrorsFrame:AddMessage("AutoLoot On", 0.0, 1.0, 0.0, 53, 3);
		end
	elseif IsControlKeyDown() and button == 'LeftButton' then
		if WatchFrame:IsShown() then
			addon:HideFrame(WatchFrame)
			addon.settings.showWatchFrame = false
		else
			addon:ShowFrame(WatchFrame)
			addon.settings.showWatchFrame = true
		end
	else
		addon:RunSlashCmd("/roll")
	end
end)

lootHotPoint:SetScript('OnEnter', function(self)
	GameTooltip:SetOwner(self, 'ANCHOR_TOP', 0, 1)
	GameTooltip:AddLine("|cffeda55fLeftClick|r toggle AutoLoot", 0.2, 1, 0.2)
	GameTooltip:AddLine("|cffeda55fRightClick|r to /roll", 0.2, 1, 0.2)
	GameTooltip:AddLine("|cffeda55fCtrl+LeftClick|r to toggle Objectives", 0.2, 1, 0.2)
	GameTooltip:Show()
end)

lootHotPoint:SetScript('OnLeave', function(self)
	GameTooltip:Hide()
end)

--[[-----------------------------------------------------------------------------
Friends list class coloring 
-------------------------------------------------------------------------------]]

hooksecurefunc('FriendsList_Update', function()
	local buttons, button = FriendsFrameFriendsScrollFrame.buttons
	local colors, CLASS = CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS, addon.CLASS
	for index = 1, #buttons do
		button = buttons[index]
		if button:IsShown() then
			if button.buttonType == FRIENDS_BUTTON_TYPE_WOW then
				local _, _, class, _, connected = GetFriendInfo(button.id)
				if connected then
					local color = colors[CLASS[class]]
					button.name:SetTextColor(color.r, color.g, color.b)
				end
			end
		else
			break
		end
	end
end)


--[[-----------------------------------------------------------------------------
Quest log expand/collapse all buttons
-------------------------------------------------------------------------------]]
local function CreateMiniButton(...)
	local button = CreateFrame('Button', nil, QuestLogFrame)
	button:SetHeight(22)
	button:SetWidth(22)
	button:SetPoint(...)
	button:RegisterForClicks('AnyUp')
	button:SetHighlightTexture([[Interface\Buttons\UI-PlusButton-Hilight]])
	return button
end

local collapseAll = CreateMiniButton('LEFT', QuestLogCount, 'RIGHT', 15, 0)
collapseAll:SetNormalTexture([[Interface\Buttons\UI-MinusButton-Up]])
collapseAll:SetPushedTexture([[Interface\Buttons\UI-MinusButton-Down]])		
collapseAll:SetScript('OnClick', function()
	CollapseQuestHeader(0)
end)

local expandAll = CreateMiniButton('LEFT', collapseAll, 'RIGHT', 3, 0)
expandAll:SetNormalTexture([[Interface\Buttons\UI-PlusButton-Up]])
expandAll:SetPushedTexture([[Interface\Buttons\UI-PlusButton-Down]])
expandAll:SetScript('OnClick', function()
	ExpandQuestHeader(0)
end)

--[[-----------------------------------------------------------------------------
Sell all grey items
-------------------------------------------------------------------------------]]
local sellGreys = CreateFrame('Button', nil, MerchantFrame, 'OptionsButtonTemplate')
sellGreys:SetPoint('TOPRIGHT', MerchantFrameCloseButton, 'BOTTOMRIGHT', -8, 4)
sellGreys:SetText("Sell Greys")
sellGreys:RegisterForClicks('AnyUp')
sellGreys:SetScript('OnClick', function()
	local total = 0
	for bag = 0, NUM_BAG_FRAMES do
		for slot = 1, GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if link then
				local _, _, itemRarity, _, _, _, _, _, _, _, itemSellPrice = GetItemInfo(link)
				local _, itemCount = GetContainerItemInfo(bag, slot)
				if itemRarity == 0 and itemSellPrice ~= 0 then
					total = total + (itemSellPrice * itemCount)
					UseContainerItem(bag, slot)
				end
			end
		end
	end
	if total ~= 0 then
		print("|cff33ff99" .. addonName .. "|r: Sold all grey items for " .. addon:MoneyToString(total))
	end
end)

--[[-----------------------------------------------------------------------------
Item border coloring for CharacterFrame and InspectFrame
-------------------------------------------------------------------------------]]
local borders, slots = { }, addon.itemSlots

local qualityColor = setmetatable({ }, { __index = function(self, key)
	local red, green, blue = key:match("(%x%x)(%x%x)(%x%x)")
	self[key] = { tonumber(red, 16) / 255, tonumber(green, 16) / 255, tonumber(blue, 16) / 255 }
	return self[key]
end })


local function UpdateItemQualityBorders(frame, _, unit)
	if frame.unit ~= unit then return end
	local border = borders[frame]
	for id = 1, #slots do
	local link = GetInventoryItemLink(unit, id)                                         -- GetInventoryItemQuality doesn't work for InspectFrame
	if link then
	local color = qualityColor[link:match("|[Cc][Ff][Ff](%x%x%x%x%x%x)")]
	border[id]:SetVertexColor(color[1], color[2], color[3])
	border[id]:Show()
	else
	border[id]:Hide()
	end

	end

end

local function GetNormalTexture(...)															-- The normal textures for InspectFrame's buttons aren't named...
	for index = 1, select('#', ...) do
		local region = select(index, ...)
		if region.GetTexture and region:GetTexture() == [[Interface\Buttons\UI-Quickslot2]] then
			return region
		end
	end
end

local function CreateBorders(frame, prefix)
	local border = { }
	for id = 1, #slots do
	local frame = _G[prefix .. slots[id]]
	local region = GetNormalTexture(frame:GetRegions())
	local texture = frame:CreateTexture(nil, 'OVERLAY')
	texture:SetTexture([[Interface\Buttons\UI-ActionButton-Border]])
	texture:SetBlendMode('ADD')
	texture:SetAlpha(0.8)
	texture:SetPoint('TOPLEFT', region, -1, 3)
	texture:SetPoint('BOTTOMRIGHT', region, 1, 0)
	border[id] = texture
	end
	borders[frame] = border
end

local function OnHide(self)
	addon.UnregisterEvent(self, 'UNIT_INVENTORY_CHANGED')
end

local function OnShow(self)
	addon.RegisterEvent(self, 'UNIT_INVENTORY_CHANGED', UpdateItemQualityBorders)
	UpdateItemQualityBorders(self, nil, self.unit)
end

local function HookFrame(frame, prefix)
	CreateBorders(frame, prefix)
	frame:HookScript('OnHide', OnHide)
	frame:HookScript('OnShow', OnShow)
end

HookFrame(CharacterFrame, 'Character')
CharacterFrame.unit = 'player'

if InspectFrame then
	HookFrame(InspectFrame, 'Inspect')
else
	addon.RegisterEvent("Features-MonitorInspectUI", 'ADDON_LOADED', function(self, event, name)
		if name ~= 'Blizzard_InspectUI' then return end
		addon.UnregisterEvent(self, event)

		HookFrame(InspectFrame, 'Inspect')
		if InspectFrame:IsShown() then
			UpdateItemQualityBorders(InspectFrame, nil, InspectFrame.unit)
		end
	end)
end


--------------------------------------
--- Macaroon default hook.
--------------------------------------

local MacTakeOver = CreateFrame("Frame", "MacTakeOver", UIParent)


MacTakeOver:RegisterEvent("ADDON_LOADED")

MacTakeOver:SetScript("OnEvent", function() 
	if addon.settings.chatInitialized == true then
		addon.settings.macaroonInitialized = true
	end
	if Macaroon and addon.settings.macaroonInitialized == false then
		addon.settings.macaroonInitialized = true
		MacaroonSavedState = {
	["cooldownAlpha"] = 1,
	["bagOffsetX"] = 0,
	["useShared"] = false,
	["bagOffsetY"] = 70,
	["timerLimit"] = 4,
	["firstRun"] = true,
	["registerForClicks"] = "AnyUp",
	["currSpec"] = 1,
	["buttonRadius"] = 87.5,
	["bars"] = {
		{
			{
				["arcLength"] = 360,
				["prowl"] = false,
				["spellGlow"] = true,
				["hotKeys"] = "",
				["copyDrag"] = false,
				["tooltips"] = true,
				["customRange"] = false,
				["hotKeyText"] = "",
				["arcStart"] = 90,
				["snapToPoint"] = false,
				["buttonList"] = {
					["homestate"] = "1;2;3;4;5;6;7;8;9;10",
				},
				["target"] = false,
				["snapTo"] = false,
				["snapToFrame"] = false,
				["stealth"] = false,
				["origID"] = 1,
				["shape"] = 1,
				["barLockShift"] = false,
				["reaction"] = false,
				["ctrl"] = false,
				["snapToPad"] = 0,
				["hidden"] = false,
				["shift"] = false,
				["y"] = 116.9545311955373,
				["x"] = 248.1222926662736,
				["name"] = "MainBar1",
				["homestate"] = true,
				["padH"] = 0,
				["barStrata"] = "MEDIUM",
				["laststate"] = "homestate",
				["alpha"] = 1,
				["alt"] = false,
				["dualSpec"] = false,
				["point"] = "BOTTOM",
				["fishing"] = false,
				["scale"] = 0.7999999999999998,
				["companion"] = false,
				["currentstate"] = "homestate",
				["barLink"] = false,
				["tooltipsCombat"] = false,
				["showstates"] = false,
				["columns"] = false,
				["autoHide"] = false,
				["barLockAlt"] = false,
				["vehicle"] = false,
				["hotKeysShown"] = true,
				["stored"] = false,
				["customNames"] = false,
				["fadeSpeed"] = 0.5,
				["tooltipsEnhanced"] = true,
				["padV"] = 0,
				["possess"] = false,
				["barLock"] = false,
				["alphaUp"] = "none",
				["group"] = false,
				["combat"] = false,
				["remap"] = false,
				["custom"] = false,
				["pagedbar"] = false,
				["showGrid"] = true,
				["barLockCtrl"] = false,
				["buttonStrata"] = "LOW",
				["stance"] = false,
			}, -- [1]
		}, -- [1]
		{
			{
				["arcLength"] = 360,
				["prowl"] = false,
				["spellGlow"] = true,
				["hotKeys"] = "",
				["copyDrag"] = false,
				["tooltips"] = true,
				["customRange"] = false,
				["hotKeyText"] = "",
				["arcStart"] = 90,
				["snapToPoint"] = false,
				["buttonList"] = {
					["homestate"] = "11;12;13;14;15;16;17;18;19;20",
				},
				["target"] = false,
				["snapTo"] = false,
				["snapToFrame"] = false,
				["stealth"] = false,
				["origID"] = 2,
				["shape"] = 1,
				["barLockShift"] = false,
				["reaction"] = false,
				["ctrl"] = false,
				["snapToPad"] = 0,
				["hidden"] = false,
				["shift"] = false,
				["y"] = 77.10642156269771,
				["x"] = 250.0914720837086,
				["name"] = "MainBar2",
				["homestate"] = true,
				["padH"] = 0,
				["barStrata"] = "MEDIUM",
				["laststate"] = "homestate",
				["alpha"] = 1,
				["alt"] = false,
				["dualSpec"] = false,
				["point"] = "BOTTOM",
				["fishing"] = false,
				["scale"] = 0.8099999999999998,
				["companion"] = false,
				["currentstate"] = "homestate",
				["barLink"] = false,
				["tooltipsCombat"] = false,
				["showstates"] = "",
				["columns"] = false,
				["autoHide"] = false,
				["barLockAlt"] = false,
				["vehicle"] = false,
				["hotKeysShown"] = true,
				["stored"] = false,
				["customNames"] = false,
				["fadeSpeed"] = 0.5,
				["tooltipsEnhanced"] = true,
				["padV"] = 0,
				["possess"] = false,
				["barLock"] = false,
				["alphaUp"] = "none",
				["group"] = false,
				["combat"] = false,
				["remap"] = false,
				["custom"] = false,
				["pagedbar"] = false,
				["showGrid"] = true,
				["barLockCtrl"] = false,
				["buttonStrata"] = "LOW",
				["stance"] = false,
			}, -- [1]
		}, -- [2]
		{
			{
				["arcLength"] = 360,
				["prowl"] = false,
				["spellGlow"] = true,
				["hotKeys"] = "",
				["copyDrag"] = false,
				["tooltips"] = true,
				["customRange"] = false,
				["hotKeyText"] = "",
				["arcStart"] = 90,
				["snapToPoint"] = false,
				["buttonList"] = {
					["homestate"] = "21;22;23;24;25;26;27;28;29;30",
				},
				["target"] = false,
				["snapTo"] = false,
				["snapToFrame"] = false,
				["stealth"] = false,
				["origID"] = 3,
				["shape"] = 1,
				["barLockShift"] = false,
				["reaction"] = false,
				["ctrl"] = false,
				["snapToPad"] = 0,
				["hidden"] = false,
				["shift"] = false,
				["y"] = 35.62762008795779,
				["x"] = 254.0300468452667,
				["name"] = "MainBar3",
				["homestate"] = true,
				["padH"] = 0,
				["barStrata"] = "MEDIUM",
				["laststate"] = "homestate",
				["alpha"] = 1,
				["alt"] = false,
				["dualSpec"] = false,
				["point"] = "BOTTOM",
				["fishing"] = false,
				["scale"] = 0.8299999999999999,
				["companion"] = false,
				["currentstate"] = "homestate",
				["barLink"] = false,
				["tooltipsCombat"] = false,
				["showstates"] = "",
				["columns"] = false,
				["autoHide"] = false,
				["barLockAlt"] = false,
				["vehicle"] = false,
				["hotKeysShown"] = true,
				["stored"] = false,
				["customNames"] = false,
				["fadeSpeed"] = 0.5,
				["tooltipsEnhanced"] = true,
				["padV"] = 0,
				["possess"] = false,
				["barLock"] = false,
				["alphaUp"] = "none",
				["group"] = false,
				["combat"] = false,
				["remap"] = false,
				["custom"] = false,
				["pagedbar"] = false,
				["showGrid"] = true,
				["barLockCtrl"] = false,
				["buttonStrata"] = "LOW",
				["stance"] = false,
			}, -- [1]
		}, -- [3]
		{
			{
				["arcLength"] = 360,
				["prowl"] = false,
				["spellGlow"] = true,
				["hotKeys"] = "",
				["copyDrag"] = false,
				["tooltips"] = true,
				["customRange"] = false,
				["hotKeyText"] = "",
				["arcStart"] = 90,
				["snapToPoint"] = false,
				["buttonList"] = {
					["vehicle1"] = "",
					["possess1"] = "",
					["homestate"] = "59;60;61;62;63;64;65;66;67;68",
					["companion1"] = "31;32;33;34;35;36;37;38;39;40",
				},
				["stored"] = false,
				["snapTo"] = false,
				["snapToFrame"] = false,
				["stealth"] = false,
				["origID"] = 4,
				["shape"] = 1,
				["barLockShift"] = false,
				["reaction"] = false,
				["ctrl"] = false,
				["snapToPad"] = 0,
				["hidden"] = false,
				["shift"] = false,
				["y"] = 116.4277150619619,
				["x"] = -260.923722280886,
				["name"] = "PetBar",
				["homestate"] = true,
				["padH"] = 0,
				["barStrata"] = "MEDIUM",
				["laststate"] = "companion1",
				["alpha"] = 1,
				["alt"] = false,
				["tooltipsCombat"] = false,
				["combat"] = false,
				["fishing"] = false,
				["group"] = false,
				["companion"] = true,
				["currentstate"] = "companion1",
				["customNames"] = false,
				["padV"] = 0,
				["showstates"] = "",
				["columns"] = false,
				["autoHide"] = false,
				["barLockAlt"] = false,
				["vehicle"] = false,
				["hotKeysShown"] = true,
				["scale"] = 0.8499999999999999,
				["dualSpec"] = false,
				["fadeSpeed"] = 0.5,
				["tooltipsEnhanced"] = true,
				["barLink"] = false,
				["possess"] = false,
				["barLock"] = false,
				["alphaUp"] = "none",
				["point"] = "BOTTOM",
				["target"] = "pet",
				["remap"] = false,
				["custom"] = false,
				["pagedbar"] = false,
				["showGrid"] = false,
				["barLockCtrl"] = false,
				["buttonStrata"] = "LOW",
				["stance"] = false,
			}, -- [1]
		}, -- [4]
		{
			{
				["arcLength"] = 360,
				["prowl"] = false,
				["spellGlow"] = true,
				["hotKeys"] = "",
				["copyDrag"] = false,
				["tooltips"] = true,
				["customRange"] = false,
				["hotKeyText"] = "",
				["arcStart"] = 90,
				["snapToPoint"] = false,
				["buttonList"] = {
					["homestate"] = "41;42;43;44;45;46;47;48;49",
				},
				["target"] = false,
				["snapTo"] = false,
				["snapToFrame"] = false,
				["stealth"] = false,
				["origID"] = 5,
				["shape"] = 1,
				["barLockShift"] = false,
				["reaction"] = false,
				["ctrl"] = false,
				["snapToPad"] = 0,
				["hidden"] = false,
				["shift"] = false,
				["y"] = 71.41417121510284,
				["x"] = -276.676797742554,
				["name"] = "MainBar4",
				["homestate"] = true,
				["padH"] = 0,
				["barStrata"] = "MEDIUM",
				["laststate"] = "homestate",
				["alpha"] = 1,
				["alt"] = false,
				["dualSpec"] = false,
				["point"] = "BOTTOM",
				["fishing"] = false,
				["scale"] = 0.8499999999999999,
				["companion"] = false,
				["currentstate"] = "homestate",
				["barLink"] = false,
				["tooltipsCombat"] = false,
				["showstates"] = "",
				["columns"] = false,
				["autoHide"] = false,
				["barLockAlt"] = false,
				["vehicle"] = false,
				["hotKeysShown"] = true,
				["stored"] = false,
				["customNames"] = false,
				["fadeSpeed"] = 0.5,
				["tooltipsEnhanced"] = true,
				["padV"] = 0,
				["possess"] = false,
				["barLock"] = false,
				["alphaUp"] = "none",
				["group"] = false,
				["combat"] = false,
				["remap"] = false,
				["custom"] = false,
				["pagedbar"] = false,
				["showGrid"] = true,
				["barLockCtrl"] = false,
				["buttonStrata"] = "LOW",
				["stance"] = false,
			}, -- [1]
		}, -- [5]
		{
			{
				["arcLength"] = 360,
				["prowl"] = false,
				["spellGlow"] = true,
				["hotKeys"] = "",
				["copyDrag"] = false,
				["tooltips"] = true,
				["customRange"] = false,
				["hotKeyText"] = "",
				["arcStart"] = 90,
				["snapToPoint"] = false,
				["buttonList"] = {
					["homestate"] = "50;51;52;53;54;55;56;57;58",
				},
				["target"] = false,
				["snapTo"] = false,
				["snapToFrame"] = false,
				["stealth"] = false,
				["origID"] = 6,
				["shape"] = 1,
				["barLockShift"] = false,
				["reaction"] = false,
				["ctrl"] = false,
				["snapToPad"] = 0,
				["hidden"] = false,
				["shift"] = false,
				["y"] = 31.81401739545321,
				["x"] = -277.6617113413033,
				["name"] = "MainBar5",
				["homestate"] = true,
				["padH"] = 0,
				["barStrata"] = "MEDIUM",
				["laststate"] = "homestate",
				["alpha"] = 1,
				["alt"] = false,
				["dualSpec"] = false,
				["point"] = "BOTTOM",
				["fishing"] = false,
				["scale"] = 0.8399999999999999,
				["companion"] = false,
				["currentstate"] = "homestate",
				["barLink"] = false,
				["tooltipsCombat"] = false,
				["showstates"] = "",
				["columns"] = false,
				["autoHide"] = false,
				["barLockAlt"] = false,
				["vehicle"] = false,
				["hotKeysShown"] = true,
				["stored"] = false,
				["customNames"] = false,
				["fadeSpeed"] = 0.5,
				["tooltipsEnhanced"] = true,
				["padV"] = 0,
				["possess"] = false,
				["barLock"] = false,
				["alphaUp"] = "none",
				["group"] = false,
				["combat"] = false,
				["remap"] = false,
				["custom"] = false,
				["pagedbar"] = false,
				["showGrid"] = true,
				["barLockCtrl"] = false,
				["buttonStrata"] = "LOW",
				["stance"] = false,
			}, -- [1]
		}, -- [6]
		{
			{
				["arcLength"] = 360,
				["prowl"] = false,
				["spellGlow"] = true,
				["hotKeys"] = "",
				["copyDrag"] = false,
				["tooltips"] = true,
				["name"] = "VehicleBar",
				["hotKeyText"] = "",
				["arcStart"] = 90,
				["snapToPoint"] = false,
				["buttonList"] = {
					["possess1"] = "",
					["vehicle1"] = "69;70;71;92;93;94;95;96;97;98",
					["homestate"] = "",
				},
				["shift"] = false,
				["snapTo"] = false,
				["padH"] = 0,
				["stealth"] = false,
				["origID"] = 7,
				["laststate"] = "homestate",
				["barLockShift"] = false,
				["reaction"] = false,
				["ctrl"] = false,
				["snapToPad"] = 0,
				["hidden"] = false,
				["stored"] = false,
				["y"] = 115.9918130609077,
				["x"] = -259.9395284377628,
				["customRange"] = false,
				["homestate"] = true,
				["snapToFrame"] = false,
				["barStrata"] = "MEDIUM",
				["shape"] = 1,
				["alpha"] = 1,
				["alt"] = false,
				["customNames"] = false,
				["combat"] = false,
				["fishing"] = false,
				["scale"] = 0.8499999999999999,
				["companion"] = false,
				["barLock"] = false,
				["barLink"] = false,
				["tooltipsCombat"] = false,
				["showstates"] = "",
				["columns"] = false,
				["autoHide"] = false,
				["barLockAlt"] = false,
				["vehicle"] = true,
				["hotKeysShown"] = true,
				["currentstate"] = "homestate",
				["dualSpec"] = false,
				["fadeSpeed"] = 0.5,
				["tooltipsEnhanced"] = true,
				["padV"] = 0,
				["possess"] = false,
				["group"] = false,
				["alphaUp"] = "none",
				["point"] = "BOTTOM",
				["target"] = false,
				["remap"] = false,
				["custom"] = false,
				["pagedbar"] = false,
				["showGrid"] = false,
				["barLockCtrl"] = false,
				["buttonStrata"] = "LOW",
				["stance"] = false,
			}, -- [1]
		}, -- [7]
		{
			{
				["arcLength"] = 360,
				["prowl"] = false,
				["spellGlow"] = true,
				["hotKeys"] = "",
				["copyDrag"] = false,
				["tooltips"] = true,
				["customRange"] = false,
				["hotKeyText"] = "",
				["arcStart"] = 90,
				["snapToPoint"] = false,
				["buttonList"] = {
					["possess1"] = "72;73;74;75;76;77;78;79;80;81",
					["homestate"] = "",
					["companion1"] = "99;100;101;102;103;104;105;106;107;108",
				},
				["stored"] = false,
				["snapTo"] = false,
				["snapToFrame"] = false,
				["stealth"] = false,
				["origID"] = 8,
				["shape"] = 1,
				["barLockShift"] = false,
				["reaction"] = false,
				["ctrl"] = false,
				["snapToPad"] = 0,
				["hidden"] = false,
				["shift"] = false,
				["y"] = 116.4289206526356,
				["x"] = -259.9393844866376,
				["name"] = "PossesionBar",
				["homestate"] = true,
				["padH"] = 0,
				["barStrata"] = "MEDIUM",
				["laststate"] = "homestate",
				["alpha"] = 1,
				["alt"] = false,
				["tooltipsCombat"] = false,
				["combat"] = false,
				["fishing"] = false,
				["group"] = false,
				["companion"] = false,
				["currentstate"] = "homestate",
				["customNames"] = false,
				["padV"] = 0,
				["showstates"] = "",
				["columns"] = false,
				["autoHide"] = false,
				["barLockAlt"] = false,
				["vehicle"] = false,
				["hotKeysShown"] = true,
				["scale"] = 0.8499999999999999,
				["dualSpec"] = false,
				["fadeSpeed"] = 0.5,
				["tooltipsEnhanced"] = true,
				["barLink"] = false,
				["possess"] = true,
				["barLock"] = false,
				["alphaUp"] = "none",
				["point"] = "BOTTOM",
				["target"] = false,
				["remap"] = false,
				["custom"] = false,
				["pagedbar"] = false,
				["showGrid"] = false,
				["barLockCtrl"] = false,
				["buttonStrata"] = "LOW",
				["stance"] = false,
			}, -- [1]
		}, -- [8]
	},
	["autocastAlpha"] = 1,
	["checkButtons"] = {
		[106] = 1,
		[104] = 1,
		[103] = 1,
	},
	["fadeSpeed"] = 0.5,
	["debug"] = {
	},
	["buttons"] = {
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 1,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 1,
				["barPos"] = 1,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [1]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 1,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 2,
				["barPos"] = 2,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [2]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 1,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 3,
				["barPos"] = 3,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [3]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 1,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 4,
				["barPos"] = 4,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [4]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 1,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 5,
				["barPos"] = 5,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [5]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 1,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 6,
				["barPos"] = 6,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [6]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 1,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 7,
				["barPos"] = 7,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [7]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 1,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 8,
				["barPos"] = 8,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [8]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 1,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 9,
				["barPos"] = 9,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [9]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 1,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 10,
				["barPos"] = 10,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [10]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 2,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 11,
				["barPos"] = 1,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [11]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 2,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 12,
				["barPos"] = 2,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [12]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 2,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 13,
				["barPos"] = 3,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [13]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 2,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 14,
				["barPos"] = 4,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [14]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 2,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 15,
				["barPos"] = 5,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [15]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 2,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 16,
				["barPos"] = 6,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [16]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 2,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 17,
				["barPos"] = 7,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [17]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 2,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 18,
				["barPos"] = 8,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [18]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 2,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 19,
				["barPos"] = 9,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [19]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 2,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 20,
				["barPos"] = 10,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [20]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 3,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 21,
				["barPos"] = 1,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [21]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 3,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 22,
				["barPos"] = 2,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [22]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 3,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 23,
				["barPos"] = 3,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [23]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 3,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 24,
				["barPos"] = 4,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [24]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 3,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 25,
				["barPos"] = 5,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [25]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 3,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 26,
				["barPos"] = 6,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [26]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 3,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 27,
				["barPos"] = 7,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [27]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 3,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 28,
				["barPos"] = 8,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [28]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 3,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 29,
				["barPos"] = 9,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [29]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 3,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 30,
				["barPos"] = 10,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [30]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 1,
				["barPos"] = 1,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "Short Circuit",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [31]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 2,
				["barPos"] = 2,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "Backfire",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [32]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 3,
				["barPos"] = 3,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "Overclock",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [33]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 4,
				["barPos"] = 4,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "Blood Pact",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [34]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 5,
				["barPos"] = 5,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "Firebolt",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [35]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 6,
				["barPos"] = 6,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "Flee",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [36]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 7,
				["barPos"] = 7,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "Singe Magic",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [37]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 8,
				["barPos"] = 8,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [38]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 9,
				["barPos"] = 9,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [39]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 10,
				["barPos"] = 10,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [40]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 5,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 11,
				["barPos"] = 1,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [41]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 5,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 42,
				["barPos"] = 2,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [42]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 5,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 43,
				["barPos"] = 3,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [43]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 5,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 44,
				["barPos"] = 4,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [44]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 5,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 45,
				["barPos"] = 5,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [45]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 5,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 46,
				["barPos"] = 6,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [46]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 5,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 47,
				["barPos"] = 7,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [47]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 5,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 48,
				["barPos"] = 8,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [48]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 5,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 49,
				["barPos"] = 9,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [49]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 6,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 50,
				["barPos"] = 1,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [50]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 6,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 51,
				["barPos"] = 2,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [51]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 6,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 52,
				["barPos"] = 3,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [52]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 6,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 53,
				["barPos"] = 4,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [53]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 6,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 54,
				["barPos"] = 5,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [54]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 6,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 55,
				["barPos"] = 6,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [55]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 6,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 56,
				["barPos"] = 7,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [56]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 6,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 57,
				["barPos"] = 8,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [57]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 6,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 58,
				["barPos"] = 9,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [58]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 59,
				["barPos"] = 1,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [59]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 60,
				["barPos"] = 2,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [60]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 61,
				["barPos"] = 3,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [61]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 62,
				["barPos"] = 4,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [62]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 63,
				["barPos"] = 5,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [63]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 64,
				["barPos"] = 6,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [64]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 65,
				["barPos"] = 7,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [65]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 66,
				["barPos"] = 8,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [66]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 67,
				["barPos"] = 9,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [67]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "homestate",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 4,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 68,
				["barPos"] = 10,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "macro",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [68]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-vehicleleave\n/click VehicleMenuBarLeaveButton",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "vehicle1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 7,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 121,
				["barPos"] = 1,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "action",
				["skincolor"] = "1;1;1;1",
				["countColor"] = "1;1;1;1",
				["macroUseNote"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [69]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-vehicleup\n/run VehicleAimIncrement()",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "vehicle1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 7,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 122,
				["barPos"] = 2,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "action",
				["skincolor"] = "1;1;1;1",
				["countColor"] = "1;1;1;1",
				["macroUseNote"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [70]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-vehicledown\n/run VehicleAimDecrement()",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "vehicle1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 7,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 123,
				["barPos"] = 3,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "action",
				["skincolor"] = "1;1;1;1",
				["countColor"] = "1;1;1;1",
				["macroUseNote"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [71]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["HHitBox"] = 0,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-possessaction\n ",
				["macroAuto"] = false,
				["scale"] = 1,
				["buffDebuffInd"] = false,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 121,
				["barPos"] = 1,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "action",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "possess1",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [72]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["HHitBox"] = 0,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-possesscancel\n/click PossessButton2",
				["macroAuto"] = false,
				["scale"] = 1,
				["buffDebuffInd"] = false,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 122,
				["barPos"] = 2,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "action",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "possess1",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [73]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["HHitBox"] = 0,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["buffDebuffInd"] = false,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 123,
				["barPos"] = 3,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "action",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "possess1",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [74]
		{
			{
				["cooldownAlpha"] = 1,
				["cooldownText"] = false,
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["buffDebuffText"] = false,
				["locked"] = false,
				["HHitBox"] = 0,
				["macro"] = "",
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["mouseAnchor"] = false,
				["hotKeyPri"] = false,
				["macroUseNote"] = false,
				["macroAuto"] = false,
				["XOffset"] = 0,
				["buffDebuffInd"] = false,
				["clickAnchor"] = false,
				["VHitBox"] = 0,
				["hovercolor"] = "0.1;0.1;1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 124,
				["barPos"] = 4,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["YOffset"] = 0,
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["fix091610"] = true,
				["hotKeyLock"] = false,
				["upClicks"] = true,
				["type"] = "action",
				["spell"] = "",
				["skincolor"] = "1;1;1;1",
				["countColor"] = "1;1;1;1",
				["macroRand"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "possess1",
				["muteSFX"] = false,
				["macroNote"] = "",
				["downClicks"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [75]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 125,
				["barPos"] = 5,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "action",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "possess1",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [76]
		{
			{
				["cooldownAlpha"] = 1,
				["cooldownText"] = false,
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["buffDebuffText"] = false,
				["locked"] = false,
				["HHitBox"] = 0,
				["macro"] = "",
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["mouseAnchor"] = false,
				["hotKeyPri"] = false,
				["macroUseNote"] = false,
				["macroAuto"] = false,
				["XOffset"] = 0,
				["buffDebuffInd"] = false,
				["clickAnchor"] = false,
				["VHitBox"] = 0,
				["hovercolor"] = "0.1;0.1;1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 126,
				["barPos"] = 6,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["YOffset"] = 0,
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["fix091610"] = true,
				["hotKeyLock"] = false,
				["upClicks"] = true,
				["type"] = "action",
				["spell"] = "",
				["skincolor"] = "1;1;1;1",
				["countColor"] = "1;1;1;1",
				["macroRand"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "possess1",
				["muteSFX"] = false,
				["macroNote"] = "",
				["downClicks"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [77]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["buffDebuffText"] = false,
				["locked"] = false,
				["HHitBox"] = 0,
				["macro"] = "#macaroon-vehicleleave\n/click VehicleMenuBarLeaveButton",
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["mouseAnchor"] = false,
				["hotKeyPri"] = false,
				["macroUseNote"] = false,
				["macroAuto"] = false,
				["XOffset"] = 0,
				["buffDebuffInd"] = false,
				["clickAnchor"] = false,
				["VHitBox"] = 0,
				["hovercolor"] = "0.1;0.1;1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 127,
				["barPos"] = 7,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["YOffset"] = 0,
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["fix091610"] = true,
				["hotKeyLock"] = false,
				["upClicks"] = true,
				["type"] = "action",
				["spell"] = "",
				["skincolor"] = "1;1;1;1",
				["countColor"] = "1;1;1;1",
				["macroRand"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "possess1",
				["muteSFX"] = false,
				["cooldownText"] = false,
				["downClicks"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [78]
		{
			{
				["cooldownAlpha"] = 1,
				["cooldownText"] = false,
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["buffDebuffText"] = false,
				["locked"] = false,
				["buffDebuffInd"] = false,
				["macro"] = "#macaroon-vehicleup\n/run VehicleAimIncrement()",
				["flyoutDock"] = false,
				["cdcolor1"] = "1;0.82;0;1",
				["bindColor"] = "1;1;1;1",
				["mouseAnchor"] = false,
				["hotKeyPri"] = false,
				["macroUseNote"] = false,
				["macroAuto"] = false,
				["XOffset"] = 0,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["VHitBox"] = 0,
				["hovercolor"] = "0.1;0.1;1;1",
				["YOffset"] = 0,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["laststate"] = "",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 128,
				["barPos"] = 8,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["muteSFX"] = false,
				["anchorDelay"] = false,
				["showstates"] = "possess1",
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["fix091610"] = true,
				["spell"] = "",
				["upClicks"] = true,
				["type"] = "action",
				["hotKeyLock"] = false,
				["skincolor"] = "1;1;1;1",
				["countColor"] = "1;1;1;1",
				["macroRand"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["countText"] = true,
				["cdcolor2"] = "1;0.1;0.1;1",
				["macroNote"] = "",
				["downClicks"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [79]
		{
			{
				["cooldownAlpha"] = 1,
				["cooldownText"] = false,
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["buffDebuffText"] = false,
				["locked"] = false,
				["buffDebuffInd"] = false,
				["macro"] = "#macaroon-vehicledown\n/run VehicleAimDecrement()",
				["flyoutDock"] = false,
				["cdcolor1"] = "1;0.82;0;1",
				["bindColor"] = "1;1;1;1",
				["mouseAnchor"] = false,
				["hotKeyPri"] = false,
				["macroUseNote"] = false,
				["macroAuto"] = false,
				["XOffset"] = 0,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["VHitBox"] = 0,
				["hovercolor"] = "0.1;0.1;1;1",
				["YOffset"] = 0,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["laststate"] = "",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 129,
				["barPos"] = 9,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["muteSFX"] = false,
				["anchorDelay"] = false,
				["showstates"] = "possess1",
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["fix091610"] = true,
				["spell"] = "",
				["upClicks"] = true,
				["type"] = "action",
				["hotKeyLock"] = false,
				["skincolor"] = "1;1;1;1",
				["countColor"] = "1;1;1;1",
				["macroRand"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["countText"] = true,
				["cdcolor2"] = "1;0.1;0.1;1",
				["macroNote"] = "",
				["downClicks"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [80]
		{
			{
				["cooldownAlpha"] = 1,
				["cooldownText"] = false,
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["buffDebuffText"] = false,
				["locked"] = false,
				["buffDebuffInd"] = false,
				["macro"] = "",
				["flyoutDock"] = false,
				["cdcolor1"] = "1;0.82;0;1",
				["bindColor"] = "1;1;1;1",
				["mouseAnchor"] = false,
				["hotKeyPri"] = false,
				["macroUseNote"] = false,
				["macroAuto"] = false,
				["XOffset"] = 0,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["VHitBox"] = 0,
				["hovercolor"] = "0.1;0.1;1;1",
				["YOffset"] = 0,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["laststate"] = "",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 130,
				["barPos"] = 10,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["muteSFX"] = false,
				["anchorDelay"] = false,
				["showstates"] = "possess1",
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["fix091610"] = true,
				["spell"] = "",
				["upClicks"] = true,
				["type"] = "action",
				["hotKeyLock"] = false,
				["skincolor"] = "1;1;1;1",
				["countColor"] = "1;1;1;1",
				["macroRand"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["countText"] = true,
				["cdcolor2"] = "1;0.1;0.1;1",
				["macroNote"] = "",
				["downClicks"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [81]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-vehicleleave\n/click VehicleMenuBarLeaveButton",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 0,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 82,
				["barPos"] = 0,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "macro",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "homestate",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = true,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [82]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-vehicleup\n/run VehicleAimIncrement()",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 0,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 83,
				["barPos"] = 0,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "macro",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "homestate",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = true,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [83]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-vehicledown\n/run VehicleAimDecrement()",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 0,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 84,
				["barPos"] = 0,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "macro",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "homestate",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = true,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [84]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 0,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 85,
				["barPos"] = 0,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "macro",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "homestate",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = true,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [85]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 0,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 86,
				["barPos"] = 0,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "macro",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "homestate",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = true,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [86]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 0,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 87,
				["barPos"] = 0,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "macro",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "homestate",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = true,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [87]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 0,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 88,
				["barPos"] = 0,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "macro",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "homestate",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = true,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [88]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 0,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 89,
				["barPos"] = 0,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "macro",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "homestate",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = true,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [89]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 0,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 90,
				["barPos"] = 0,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "macro",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "homestate",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = true,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [90]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 0,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 91,
				["barPos"] = 0,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "macro",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "homestate",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = true,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [91]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["buffDebuffText"] = false,
				["stored"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 7,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 124,
				["barPos"] = 4,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "action",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "vehicle1",
				["XOffset"] = 0,
				["downClicks"] = false,
				["mouseAnchor"] = false,
				["locked"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [92]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 7,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 125,
				["barPos"] = 5,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "action",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "vehicle1",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [93]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["buffDebuffText"] = false,
				["stored"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 7,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 126,
				["barPos"] = 6,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "action",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "vehicle1",
				["XOffset"] = 0,
				["downClicks"] = false,
				["mouseAnchor"] = false,
				["locked"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [94]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-vehicleleave\n/click VehicleMenuBarLeaveButton",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 7,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 95,
				["barPos"] = 7,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "macro",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "vehicle1",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [95]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-vehicleup\n/run VehicleAimIncrement()",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 7,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 96,
				["barPos"] = 8,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "macro",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "vehicle1",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [96]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-vehicledown\n/run VehicleAimDecrement()",
				["macroAuto"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 7,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 97,
				["barPos"] = 9,
				["scale"] = 1,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["macroUseNote"] = false,
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["spell"] = "",
				["type"] = "macro",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "vehicle1",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [97]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["stored"] = false,
				["fix091610"] = true,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["buffDebuffInd"] = false,
				["bindColor"] = "1;1;1;1",
				["laststate"] = "",
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["VHitBox"] = 0,
				["cooldownText"] = false,
				["muteSFX"] = false,
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 7,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 130,
				["barPos"] = 10,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "action",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "vehicle1",
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["locked"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [98]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-vehicleleave\n/click VehicleMenuBarLeaveButton",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 1,
				["barPos"] = 1,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "Short Circuit",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [99]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-vehicleup\n/run VehicleAimIncrement()",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 2,
				["barPos"] = 2,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "Backfire",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [100]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "#macaroon-vehicledown\n/run VehicleAimDecrement()",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 3,
				["barPos"] = 3,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "Overclock",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [101]
		{
			{
				["cooldownAlpha"] = 1,
				["macroNote"] = "",
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["locked"] = false,
				["VHitBox"] = 0,
				["macroRand"] = false,
				["flyoutDock"] = false,
				["laststate"] = "",
				["bindColor"] = "1;1;1;1",
				["buffDebuffInd"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["scale"] = 1,
				["HHitBox"] = 0,
				["clickAnchor"] = false,
				["fix091610"] = true,
				["muteSFX"] = false,
				["showstates"] = "companion1",
				["clearerrors"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 4,
				["barPos"] = 4,
				["hovercolor"] = "0.1;0.1;1;1",
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["cdcolor2"] = "1;0.1;0.1;1",
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["YOffset"] = 0,
				["spell"] = "Blood Pact",
				["upClicks"] = true,
				["hotKeyLock"] = false,
				["type"] = "pet",
				["macroUseNote"] = false,
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["cooldownText"] = false,
				["XOffset"] = 0,
				["buffDebuffText"] = false,
				["mouseAnchor"] = false,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [102]
		{
			{
				["cooldownAlpha"] = 1,
				["cooldownText"] = false,
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["mouseAnchor"] = false,
				["fix091610"] = true,
				["HHitBox"] = 0,
				["flyoutDock"] = false,
				["macroUseNote"] = false,
				["bindColor"] = "1;1;1;1",
				["clearerrors"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["laststate"] = "",
				["buffDebuffInd"] = false,
				["clickAnchor"] = false,
				["VHitBox"] = 0,
				["locked"] = false,
				["macroNote"] = "",
				["macroRand"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 5,
				["barPos"] = 5,
				["XOffset"] = 0,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["YOffset"] = 0,
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["cdcolor2"] = "1;0.1;0.1;1",
				["hotKeyLock"] = false,
				["upClicks"] = true,
				["type"] = "pet",
				["spell"] = "Firebolt",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffDebuffText"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "companion1",
				["muteSFX"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["scale"] = 1,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [103]
		{
			{
				["cooldownAlpha"] = 1,
				["cooldownText"] = false,
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["mouseAnchor"] = false,
				["fix091610"] = true,
				["HHitBox"] = 0,
				["flyoutDock"] = false,
				["macroUseNote"] = false,
				["bindColor"] = "1;1;1;1",
				["clearerrors"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["laststate"] = "",
				["buffDebuffInd"] = false,
				["clickAnchor"] = false,
				["VHitBox"] = 0,
				["locked"] = false,
				["macroNote"] = "",
				["macroRand"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 6,
				["barPos"] = 6,
				["XOffset"] = 0,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["YOffset"] = 0,
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["cdcolor2"] = "1;0.1;0.1;1",
				["hotKeyLock"] = false,
				["upClicks"] = true,
				["type"] = "pet",
				["spell"] = "Flee",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffDebuffText"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "companion1",
				["muteSFX"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["scale"] = 1,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [104]
		{
			{
				["cooldownAlpha"] = 1,
				["cooldownText"] = false,
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["mouseAnchor"] = false,
				["fix091610"] = true,
				["HHitBox"] = 0,
				["flyoutDock"] = false,
				["macroUseNote"] = false,
				["bindColor"] = "1;1;1;1",
				["clearerrors"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["laststate"] = "",
				["buffDebuffInd"] = false,
				["clickAnchor"] = false,
				["VHitBox"] = 0,
				["locked"] = false,
				["macroNote"] = "",
				["macroRand"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 7,
				["barPos"] = 7,
				["XOffset"] = 0,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["YOffset"] = 0,
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["cdcolor2"] = "1;0.1;0.1;1",
				["hotKeyLock"] = false,
				["upClicks"] = true,
				["type"] = "pet",
				["spell"] = "Singe Magic",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffDebuffText"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "companion1",
				["muteSFX"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["scale"] = 1,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [105]
		{
			{
				["cooldownAlpha"] = 1,
				["cooldownText"] = false,
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["mouseAnchor"] = false,
				["fix091610"] = true,
				["HHitBox"] = 0,
				["flyoutDock"] = false,
				["macroUseNote"] = false,
				["bindColor"] = "1;1;1;1",
				["clearerrors"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["laststate"] = "",
				["buffDebuffInd"] = false,
				["clickAnchor"] = false,
				["VHitBox"] = 0,
				["locked"] = false,
				["macroNote"] = "",
				["macroRand"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 8,
				["barPos"] = 8,
				["XOffset"] = 0,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["YOffset"] = 0,
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["cdcolor2"] = "1;0.1;0.1;1",
				["hotKeyLock"] = false,
				["upClicks"] = true,
				["type"] = "pet",
				["spell"] = "",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffDebuffText"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "companion1",
				["muteSFX"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["scale"] = 1,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [106]
		{
			{
				["cooldownAlpha"] = 1,
				["cooldownText"] = false,
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["mouseAnchor"] = false,
				["fix091610"] = true,
				["HHitBox"] = 0,
				["flyoutDock"] = false,
				["macroUseNote"] = false,
				["bindColor"] = "1;1;1;1",
				["clearerrors"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["laststate"] = "",
				["buffDebuffInd"] = false,
				["clickAnchor"] = false,
				["VHitBox"] = 0,
				["locked"] = false,
				["macroNote"] = "",
				["macroRand"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 9,
				["barPos"] = 9,
				["XOffset"] = 0,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["YOffset"] = 0,
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["cdcolor2"] = "1;0.1;0.1;1",
				["hotKeyLock"] = false,
				["upClicks"] = true,
				["type"] = "pet",
				["spell"] = "",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffDebuffText"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "companion1",
				["muteSFX"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["scale"] = 1,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [107]
		{
			{
				["cooldownAlpha"] = 1,
				["cooldownText"] = false,
				["hotKeys"] = ":",
				["hotKeyText"] = ":",
				["spellCounts"] = false,
				["downClicks"] = false,
				["mouseAnchor"] = false,
				["fix091610"] = true,
				["HHitBox"] = 0,
				["flyoutDock"] = false,
				["macroUseNote"] = false,
				["bindColor"] = "1;1;1;1",
				["clearerrors"] = false,
				["hotKeyPri"] = false,
				["macro"] = "",
				["macroAuto"] = false,
				["laststate"] = "",
				["buffDebuffInd"] = false,
				["clickAnchor"] = false,
				["VHitBox"] = 0,
				["locked"] = false,
				["macroNote"] = "",
				["macroRand"] = false,
				["bdcolor1"] = "0;0.82;0;1",
				["cdcolor1"] = "1;0.82;0;1",
				["bar"] = 8,
				["comboCounts"] = false,
				["macroName"] = "",
				["action"] = 0,
				["barPos"] = 10,
				["XOffset"] = 0,
				["rangeInd"] = true,
				["bdcolor2"] = "1;0.1;0.1;1",
				["YOffset"] = 0,
				["anchorDelay"] = false,
				["countText"] = true,
				["anchoredBar"] = false,
				["debuffcolor"] = "0.8;0;0;1",
				["bindText"] = true,
				["cdcolor2"] = "1;0.1;0.1;1",
				["hotKeyLock"] = false,
				["upClicks"] = true,
				["type"] = "pet",
				["spell"] = "",
				["countColor"] = "1;1;1;1",
				["skincolor"] = "1;1;1;1",
				["buffDebuffText"] = false,
				["buffcolor"] = "0;0.8;0;1",
				["alpha"] = 1,
				["equipcolor"] = "0.1;1;0.1;1",
				["rangecolor"] = "0.7;0.15;0.15;1",
				["showstates"] = "companion1",
				["muteSFX"] = false,
				["hovercolor"] = "0.1;0.1;1;1",
				["scale"] = 1,
				["stored"] = false,
				["macroIcon"] = 1,
				["fix100310"] = true,
			}, -- [1]
			[0] = false,
		}, -- [108]
	},
	["sharedProfile"] = false,
	["bfskin"] = {
	},
	["bagScale"] = 1,
	["rightClickTarget"] = false,
	["snapToTol"] = 28,
	["focusCast"] = false,
	["selfCast"] = false,
	["options"] = {
	},
	["panelScale"] = 1,
	["buttonLoc"] = {
		-0.85, -- [1]
		-111.45, -- [2]
	},
	["throttle"] = 0.2,
	["fix100310"] = true,
}
	end
end)